classdef BSplines
% Implements functionality for BSpline interpolation.
%
% BSplines Properties:
% xI - Positions of the data sites.
% yI - Function values at the given data sites.
% order - Order of the considered spline functions.
% knots - Knots needed for the construction of a B-Spline basis.
%
% BSplines Methods:
% Basis - Evaluates B-Spline basis function.
% BSplines - Constructor.
% CollocMatrix - Compute the collocation matrix.
% ConstructKnots - Returns a correct knot sequence for the B-Splines.
% DiffBasis - Differentiate B-Spline basis functions.
% Interpolate - Interpolate the data.
% NumBSplines - Number of Basis functions for a given knot sequence.
% NumKnots - Number of knots necessary to solve the interpolation problem.
%
% Example
% Set up cubic BSplines on the interval [-pi,pi].
% b = BSplines(linspace(-pi,pi,10),4)
%
% See also CardinalSplines

% Copyright 2011, 2012, 2013 Laurent Hoeltgen <hoeltgman@gmail.com>
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 3 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
% MA 02110-1301, USA.

    properties
        xI    % Positions of the data sites.
        yI    % Function values at the given data sites.
        order % Order of the considered spline functions.
        knots % Knots needed for the construction of a B-Spline basis.
    end
    
    methods
        function obj = BSplines(yI,order,xI,knots)
            
            narginchk(2,4);
            nargoutchk(0,1);

            % yI should either be a vector (in which case it will be saved as a
            % row vector) or a matrix. If xI is specified, then the number of
            % columns should coincide with the number of data sites in xI.
            obj.yI = yI;
            
            obj.order = order;            
    
            if nargin > 2
                obj.xI = xI;
                
                % Check that data is complete.
                assert(length(obj.xI) == length(obj.yI), 'BSplines:BadVal', ...
                    'Number of data sites and function values do not coincide.');
            else
                obj.xI = 1:length(transpose(obj.yI));
            end

            if nargin > 3
                obj.knots = knots;
            else
                obj.knots = obj.ConstructKnots();
            end 
        end % Constructor
        
        function obj = set.xI(obj,val)
            assert(isvector(val), 'BSplines:BadVal', ...
                'Argument must be a vector.');
                if isrow(val)
                    obj.xI = val;
                else
                    obj.xI = transpose(val);
                end
        end
        
        function obj = set.yI(obj,val)
            assert(isvector(val)||ismatrix(val), 'BSplines:BadVal', ...
                'Argument must be a vector or a matrix.');
            if isrow(val)
                obj.yI = val;
            elseif iscolumn(val)
                obj.yI = transpose(val);
            else
                obj.yI = val;
            end
        end
        
        function obj = set.order(obj,val)
            assert(isscalar(val)&&(val>0), 'BSplines:BadVal', ...
                'Argument must be a positive scalar.');
            obj.order = uint32(val);
        end
        
        function obj = set.knots(obj,val)
            assert(isvector(val), 'BSplines:BadVal', ...
                'Argument must be a vector.');
            if isrow(val)
                obj.knots = val;
            else
                obj.knots = transpose(val);
            end
        end
        
        function val = get.xI(obj)
            val = obj.xI;
        end
        
        function val = get.yI(obj)
            val = obj.yI;
        end
        
        function val = get.order(obj)
            val = obj.order;
        end
        
        function val = get.knots(obj)
            val = obj.knots;
        end
        
    end % Methods Block
    
    methods (Static = true)
        y = Basis(x,num,order,knot)
        y = DiffBasis(x,num,order,knot,difforder)
        knots = NumKnots(order,cond)
        number = NumBSplines(order,knots)
    end
end % Classdef
