function [t n] = ConstructKnots(obj,v)
% Constructs a valid knot sequence.
% Constructs a valid knot sequence for interpolation using the smoothness
% conditions specified by the vector v (if specified). The construction is based
% on the suggested scheme found in "A practical guide to splines", Carl de Boor,
% Ch XIII, p.182 (if the order is even). For odd orders, the number of returned
% knots is correct, however it is unknown whether the sequence is correct.
% Furthermore the second argument is always ignored when the order is odd.
%
% Explanation:
% The choice of t translates the desired amount of smoothness at a break (as
% specified by v) into a corresponding number of knots at that site, with fewer
% knots corresponding to more continuity conditions, in such a way that always
%
% number of cont. conditions at x + number of knots at x = order
%
% Thus a order-fold knot at a site corresponds to no continuity condition at
% that site while, no knot at a site enforces order continuity conditions there.
%
% Example
% Construct a reasonable knot sequence for cubic spline interpolation.
% b = BSplines(sin(linspace(-1,1,10)),4);
%
% See also BSplines

% Copyright 2011, 2012, 2013 Laurent Hoeltgen <hoeltgman@gmail.com>
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 3 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
% MA 02110-1301, USA.

% Last revision on: 24.04.2013 09:55

narginchk(0,1);
nargoutchk(0,2);

x = obj.xI;
k = obj.order;

if mod(k,2) == 0
    if nargin == 1
        v = double((k-1))*ones(1,length(x)-2);
        v(1:(k/2-1)) = k;
        v((end-k/2+2):end) = k;
    end

    assert(isrow(v) , 'BSplines:BadVal' , ...
        'Input parameter should be a row vector');
    assert(length(x) == length(v) + 2)
    assert( any(v>k) == false );
    
    t = [];
    for i=1:length(v)
        t = [ t , x(i+1)*ones(1,k-v(i)) ];
    end
    n = k + sum(double(k)-v);
    t = [ x(1)*ones(1,k) , t , x(end)*ones(1,k) ];
else
    warning('BSplines:BadVal' , ...
        'Knots might be set incorrectly for odd orders.');
    if nargin == 2
        warning('BSplines:BadParam' , ...
            'Second argument cannot be used in combination with odd orders.');
    end
    m = linspace(x(1),x(end),BSplines.NumKnots(k,length(x))-2*k+2);
    t = [ x(1)*ones(1,k) , m(2:(end-1)) , x(end)*ones(1,k) ];
end
end
