function y = Interpolate(obj,x,w)
% Interpolate using B-Splines.
% Perform B-Spline based interpolation.
%
% Interpolate returns the interpolated values at
% positions found in x. t is an optional parameter specifying the knot sequence
% to be used. If yI contains several values per data site (i.e. yI is a matrix
% and not just a row vector), then these are simply averaged using the usual
% arithmetic mean. A 6-th optional parameter w can be specified that provides
% the weights to be used for the averaging. w should be a matrix of the same
% size as yI then.
%
% xI : break points for the spline Interpolation. [Row vector]
% yI : samples at the break points. [Row vector or Matrix (multiple samples)]
%      Note that yI must have the same number of columns as xI.
% k  : order of the Spline. e.g. 4 for cubic splines. [Scalar]
% x  : interpolation sites. [Row vector] The values in x should all lie in the
%      interval [ min(xI) , max(xI) ].
% t  : knot vector. (optional) Remember that for n data sites and interpolation
%      order k, there must be n+k knots to perform the interpolation.
% w  : weights for the averaging of multiple samples. [Matrix] (optional) If w
%      is specified, then it must have the same dimensions as yI.
%
% Example
% Interpolate the function sin(x) on the interval [-1,1].
% b = BSplines(sin(linspace(-1,1,10)),4);
% b.Interpolate(linspace(-1,1,1000));
%
% See also BSplines, ConstructKnots

% Copyright 2011, 2012, 2013 Laurent Hoeltgen <hoeltgman@gmail.com>
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 3 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
% MA 02110-1301, USA.

% Last revision on: 24.04.2013 09:55

narginchk(2,3);
nargoutchk(0,1);

xI = obj.xI;
yI = obj.yI;
n = length(xI);
k = obj.order;
t = obj.knots;

% If we have several samples at each sampling point, then we simply average
% them. If the parameter w is given, it is used as a weight. w must have the
% same size as yI.
if ismatrix(yI) && ~isrow(yI)
    if nargin < 3
        yI = mean(yI);
    else
        assert(isequal(size(yI),size(w)));
        yI = sum(yI.*w)./sum(w);
    end
end

% Construct a valid Knot sequence corresponding to the sampling points. The
% construction is rather simple. We use k times the first and last break and
% complete the knot sequence with sufficiently elements from xI.

% Check if we can generate a complete B-Spline Basis. If problem is
% overdetermined, we will use a least squares approach and emit a warning.
% If the problem is underdetermined, we abort the computation because it is
% impossible to know if the found solution makes sense or not.
if BSplines.NumBSplines(k,length(t)) < n
    warning('BSplines:NoBasis', ...
        'Not enough knots to generate a BSpline basis.');
elseif BSplines.NumBSplines(k,length(t)) > n
    error('BSplines:NoBasis', ...
        'Too many knots. Solution is not unique. Aborting.');
end

% Set up the collocation Matrix.
B = obj.CollocMatrix(t);

% Solve the corresponding linear system. Note that mldivide automatically
% performs a least square fit if B is not a square matrix.
c = B\yI';

% Compute the interpolated values at the positions given by x.
y = zeros(size(x));
N = BSplines.NumBSplines(k,length(t));
for i = 1:N
    y = y + c(i)*BSplines.Basis(x,i,k,t);
end
end

