%% BSpline Class Demonstrations.
%  This file provides examples and demonstrations on how to use the BSplines
%  class.
%
% Copyright 2011 Laurent Hoeltgen <hoeltgman@gmail.com>
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 3 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
% MA 02110-1301, USA.

%% Various interpolations.
%  A simple example that shows how to perform interpolation with different 
%  approximation orders.

clear all;

f = @(x) sin(x)+log(1+x^2);
xI = linspace(-pi,pi,10);
yI = arrayfun(f,xI);

% Linear interpolation.
b1 = BSplines(yI,2,xI);
% Cubic interpolation.
b2 = BSplines(yI,4,xI);
% Quintic interpolation.
b3 = BSplines(yI,6,xI);

x = linspace(-pi,pi,1000);
y1 = b1.Interpolate(x);
y2 = b2.Interpolate(x);
y3 = b3.Interpolate(x);

figure(101); plot(xI,yI,'*r',x,y1)
title('Linear Interpolation');
figure(102); plot(xI,yI,'*r',x,y2)
title('Cubic Interpolation');
figure(103); plot(xI,yI,'*r',x,y3)
title('Quintic Interpolation');

%% Determine the number of knots necessary to perform interpolation.
%  The function NumKnots can be used to determine the necessary number of knots
%  to solve the interplation problem.

clear all;

order = 3;
numcond = 14;
BSplines.NumKnots(order,numcond)

%% Determine the number of B-Splines one can generate.
%  NumBSplines returns the number of B-Spline Basis functions that can be
%  generated for a given knot sequence.
clear all;

order = 4;
numknots = 16;
BSplines.NumBSplines(order,numknots)

%% Compute a valid knot sequence and the collocation matrix.

f = @(x) sin(x)+log(1+x^2);
xI = linspace(-pi,pi,8);
yI = arrayfun(f,xI);
order = 4;
b = BSplines(yI,order,xI);

b.ConstructKnots()
b.CollocMatrix()

%% Perform approximation with insufficient knots.
%  If too few knots are provided, then the linear system with the collocation
%  matrix is solved in the least squares sense. The obtained interpolating
%  function will not necessarily go through the data samples then.

clear all;

f = @(x) sin(x)+log(1+x^2);
xI = linspace(-pi,pi,10);
yI = arrayfun(f,xI);
order = 4;
b = BSplines(yI,order,xI);
b.knots = [-pi,-pi,-pi,-pi,pi,pi,pi,pi];

x = linspace(-pi,pi,1000);
y = b.Interpolate(x);

figure(104); plot(xI,yI,'*r',x,y)
title('Approximation with insufficent knots.');

%% Average multiple sample values with a weight before Interpolating.
%  If more than 1 sample is provided per data site, these can be averaged using
%  a weight matrix. If no weights are specified, then we simply take the
%  arithmetic mean.

clear all;

f = @(x) sin(x)+log(1+x.^2);
xI = linspace(-pi,pi,10);
yI = [ f(xI) ; ...
       f(xI) + randn(size(xI)).*0.10 ; ...
       f(xI) + randn(size(xI)).*0.15 ; ...
       f(xI) + randn(size(xI)).*0.20 ];
   
w = [ 4*ones(size(xI)) ; 3*ones(size(xI)) ; 2*ones(size(xI)) ; ones(size(xI)) ];
   
order = 6;
b = BSplines(yI,order,xI);

x = linspace(-pi,pi,1000);
y = b.Interpolate(x,w);
   
figure(105); plot(xI,yI,'*r',x,y)
title('Interpolation with averaging the samples');
figure(106); plot(xI,sum(yI.*w)./sum(w),'*r',x,y)
title('Curve interpolates averaged samples.');


%% Average knot weights and perform least squares interpolation.
%  Averaging the samples can also be combined with the least squares solver.

clear all;

f = @(x) sin(x)+log(1+x.^2);
xI = linspace(-pi,pi,10);
yI = [ f(xI) ; ...
       f(xI) + randn(size(xI)).*0.10 ; ...
       f(xI) + randn(size(xI)).*0.15 ; ...
       f(xI) + randn(size(xI)).*0.20 ];

w = [ 4*ones(size(xI)) ; 3*ones(size(xI)) ; 2*ones(size(xI)) ; ones(size(xI)) ];
   
order = 6;
b = BSplines(yI,order,xI);
b.knots = [-pi,-pi,-pi,-pi,-pi,-pi,pi,pi,pi,pi,pi,pi];

x = linspace(-pi,pi,1000);
y = b.Interpolate(x,w);
   
figure(107); plot(xI,yI,'*r',x,y)
title('Approximation and averaging weights');
figure(108); plot(xI,sum(yI.*w)./sum(w),'*r',x,y)
title('Curve approximates averaged samples.');


%% Evaluate B-Spline Basis functions and their derivatives.

clear all;

f = @(x) sin(x)+log(1+x^2);
xI = linspace(-pi,pi,10);
yI = arrayfun(f,xI);
order = 4;
b = BSplines(yI,order,xI);

x = linspace(-pi,pi,1000);
y = transpose(cell2mat(arrayfun( @(i)BSplines.Basis(x',i,order,b.knots), ...
    1:BSplines.NumBSplines(order,length(b.knots)),'UniformOutput',false)));

yp = transpose( ...
    cell2mat(arrayfun( @(i)BSplines.DiffBasis(x',5,order,b.knots,i), ...
    0:3,'UniformOutput',false)));

figure(109); plot(x,y)
title('Cubic B-Spline basis functions.');

figure(110); plot(x,yp)
title('Derivatives of the 5th B-Spline function');
legend('5th B-Spline', '1st derivative', '2nd derivative', '3rd derivative');