function y = ClassicInterpolation(x,yI,f,xI,classic)
% ClassicInterpolation
% Perform classical interpolation. xI must be a uniform grid (a vector). In case
% the gridsize is different from 1, it will be scaled accordingly.
%
% Example
% f = @(x) (-0.5<=x)&&(x<=0.5)
% ClassicInterpolation(0.1,-3:3,sin(-3:3),f)
%
% See also interp

% Copyright 2011, 2012, 2013 Laurent Hoeltgen <hoeltgman@gmail.com>
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 3 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
% MA 02110-1301, USA.

% Last revision on: 24.04.2013 09:55

narginchk(3,5);
nargoutchk(0,1);

assert(isrow(yI),'Interpolation:BadArg', ...
    'Second argument must be a row vector.');
assert(isa(f,'function_handle'),'Interpolation:BadArg', ...
    'Third argument must be a function handle.');

if nargin == 3
    xI = 1:length(yI);
end

if nargin >= 4
    assert(isrow(xI),'Interpolation:BadArg', ...
        'Last argument must be a row vector.');
    assert(isequal(size(yI),size(xI)),'Interpolation:BadArg', ...
        'Second and fourth argument must have same size.');
end

if nargin < 5
    classic = true;
end

if (abs(f(0)-1) > 1e-10)&&(classic == true)
    warning('Interpolation:BadArg', ...
        'Warning f(0) does not equal 1.');
end

assert( max(x) <= max(xI),'Interpolation:BadArg', ...
    'Interpolation point outside of domain.');
assert( min(x) >= min(xI),'Interpolation:BadArg', ...
    'Interpolation point outside of domain.');

dx = diff(xI);
assert( all( abs(dx-dx(1))<=1e-10 ),'Interpolation:BadArg', ...
    'Data sites do not form a uniform grid.');

h = dx(1);
if classic == true
    y = arrayfun(@(t)sum(yI.*f((t-xI)/h)),x);
else
    B = CollocMatrix(xI,f);
    c = transpose(B\transpose(yI));
    y = arrayfun( @(t)sum(c.*f((t-xI)/h)) , x );
end
end

function B = CollocMatrix(xI,f)
n = length(xI);
psi = @(x,y) f(x-y);
aa = kron(ones(1,n),[1:n]');
bb = kron(ones(n,1),[1:n]);
B = arrayfun(psi,aa,bb);
end
