function z = ClassicInterpolation2D(x,y,zI,fx,fy,xI,yI,classic)
% ClassicInterpolation2D
% Performs Classic interpolation in 2D on a uniform grid of data sites.
% x is a row vector with the x-coordinates of the interpolation points.
% y is a row vector with the y-coordinates of the interpolation points. It must
% have the same size as x.
% zI are the sample values. zI should be a matrix.
% f is a function handle to the synthesis function to be used.
% xI is a row vector specifying the x-coordinates of the data sites.
% yI is a row vector specifying the y-coordinates of the data sites.
% classic is boolean value indicating whether we should apply classical
% interpolation or not.
%
% Example
%xI = linspace(-pi,pi,10);
% yI = linspace(0,2*pi,15);
% fh = @(x,y) sin(x).*(1+cos(y)).^2;
% [XI YI] = meshgrid(xI,yI);
% zI = fh(XI,YI);
% figure(1001); surf(XI,YI,zI);
% x = kron(linspace(-pi,pi,40),ones(1,60));
% y = kron(ones(1,40),linspace(0,2*pi,60));
% phi = @(x) (1-abs(x)).*(abs(x)<=1);
% z = ClassicInterpolation2D(x,y,zI,phi,phi,xI,yI,false);
% Z = reshape(z,60,40);
% [XX YY] = meshgrid(linspace(-pi,pi,40),linspace(-pi,pi,60));
% figure(1002); surf(XX,YY,Z);
%
% See also ClassicInterpolation

% Copyright 2011,  2012, 2013 Laurent Hoeltgen <hoeltgman@gmail.com>
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 3 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
% MA 02110-1301, USA.

% Last revision on: 24.04.2013 09:55

narginchk(5,8);
nargoutchk(0,1);

assert(isrow(x),'Interpolation:BadArg', ...
    'First argument must be a row vector.');
assert(isrow(y),'Interpolation:BadArg', ...
    'Second argument must be a row vector.');
assert(length(x)==length(y),'Interpolation:BadArg.', ...
    'First two arguments must have same size.');
assert(ismatrix(zI),'Interpolation:BadArg', ...
    'Third argument must be a row vector.');
assert(isa(fx,'function_handle'),'Interpolation:BadArg', ...
    'Fourth argument must be a function handle.');
assert(isa(fy,'function_handle'),'Interpolation:BadArg', ...
    'Fifth argument must be a function handle.');
if nargin > 5
    assert(isrow(xI),'Interpolation:BadArg', ...
        'Fifth argument must be a row vector.');
    if nargin > 6
        assert(isrow(yI),'Interpolation:BadArg', ...
            'Sixth argument must be a row vector.');
    end
    if nargin == 8
        assert(islogical(classic),'Interpolation:BadArg', ...
            'Last argument must be a logical.');
    end
else
    [ r c ] = size(zI);
    xI = 1:c;
    yI = 1:r;
end
assert(isequal(size(zI),[length(yI),length(xI)]),'Interpolation:BadArg', ...
    'Size of Matrix with sampling values is incorrect.');

nx = length(xI);
ny = length(yI);

% Interpolate each row in x-direction.
P = zeros(ny,length(x));
for i = 1:ny
    P(i,:) = ClassicInterpolation(x,zI(i,:),fx,xI,classic);
end

% Interpolate the columns specified by x in y-direction.
nnx = length(x);
Q = zeros(1,nnx);
for i = 1:nnx
    Q(i) = ClassicInterpolation(y(i),P(:,i)',fy,yI,classic);
end

% Now interpolate the data set in first in each row and then in each column. At
% the end, we will average both results.

% Interpolate each column in y-direction
R = zeros(nx,length(y));
for i = 1:nx
    R(i,:) = ClassicInterpolation(y,zI(:,i)',fy,yI,classic);
end

% Interpolate the rows specified by y in x-direction
nny = length(y);
S = zeros(1,nny);
for i = 1:nny
    S(i) = ClassicInterpolation(x(i),R(:,i)',fx,xI,classic);
end

% Return the arithmetic of both results. This should make the result less
% dependant of the choice along which dimension we interpolate first.

z = (Q+S)/2;

end
